import {
  Metadata, IDictionary, Datasource, IEventParam, Loader, IDatasource, DatasourceFactory, Config,
} from '@zeedhi/core';
import {
  Button, LoadingService, GridEditable, DialogService, Card, CollapseCard, Col,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pme02600ReferenciaExportacao {
  // private detailIsVisible: boolean;
  
  private isFormatting = false;
  
  private utilities = Loader.getInstance('Utilities');
  
  private gridEditController = Loader.getInstance('GridEditableController');
  
  private crudController = Loader.getInstance('CrudController');
  
  private appController = Loader.getInstance('AppController');
  
  private collapseCardInUse = '';
  
  // public gridColsReturnValue(): number {
  //   return this.detailIsVisible ? 6 : 12;
  // }
  
  // get detailIsVisible() {
  //   const colDetail = Metadata.getInstance<Col>(`colDetail-${this.collapseCardInUse}`);
  //   return (colDetail?.cols === 6);
  // }
  
  set detailIsVisible(value: boolean) {
    try {
      const colReferencia = Metadata.getInstance<Col>(`colReferencia-${this.collapseCardInUse}`);
      const colDetail = Metadata.getInstance<Col>(`colDetail-${this.collapseCardInUse}`);
      const detailCard = Metadata.getInstance<Card>(`detailCard-${this.collapseCardInUse}`);
      if (value) {
        colReferencia.cols = 6;
        colDetail.cols = 6;
        detailCard.isVisible = true;
      } else {
        colReferencia.cols = 12;
        colDetail.cols = 12;
        detailCard.isVisible = false;
      }
    } catch (error) {
      // console.warn(error);
    }
  }

  public setCollapseCardInUse({ component }: IEventParam<CollapseCard>) {
    this.collapseCardInUse = component.userProperties?.userProperties;
  }

  public closeCardDetail() {
    const detail = Metadata.getInstance<GridEditable>(`gridDetail-${this.collapseCardInUse}`);
    if (detail?.editing) this.gridEditController.cancelEditing({ component: detail });
    this.detailIsVisible = false;
  }
  
  private async getTotalIntegraCardPme() {
    const ds = DatasourceFactory.factory({
      uniqueKey: 'CDINTEGRA',
      route: '/find/integraCardPme',
      type: 'rest',
      loadAll: true,
    });
    LoadingService.show(' ');
    await ds.get();
    LoadingService.hide();
    return ds.total;
  }

  public async saveRef({ component }: IEventParam<Button>) {
    const grid: GridEditable = component.parent as GridEditable;
    const editedRows: IDictionary = grid?.getEditedRows(false, true);

    if (grid?.isGridValid(true)) {
      if (editedRows.length > 0) {
        if (grid.name === `gridPrincipal-${this.collapseCardInUse}`) {
          const hasNew = editedRows.some((el: any) => el.__is_new);
          let total: number;
          if (hasNew) total = await this.getTotalIntegraCardPme();
          editedRows.forEach((row: IDictionary) => {
            if (row.__is_new) {
              total += 1;
              row.NRINTEGRACARDPME = total;
              row.IDTIPO = grid?.datasource?.filter[0]?.value;
            }
          });
          const regexFind = /\/find/gm;
          const datasourceName = (grid?.datasource as IDatasource).route.replace(regexFind, '');
          LoadingService.show(' ');
          this.crudController.saveRows(editedRows, datasourceName, grid).then(() => {
            this.gridEditController.defaultState(grid);
            // gridServicoDetail.datasource.get();
            this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
        } else if (grid.name === `gridDetail-${this.collapseCardInUse}`) {
          const gridPrincipal = Metadata.getInstance<GridEditable>(`gridPrincipal-${this.collapseCardInUse}`);
          const editedRowsToSend: any = [];
          Object.keys(editedRows[0]).forEach((key: any) => {
            if (Array.isArray(editedRows[0][key])) {
              editedRows[0][key].forEach((value: any, idx: number) => {
                if (editedRowsToSend[idx]) editedRowsToSend[idx][key] = value;
                else {
                  editedRowsToSend[idx] = {};
                  editedRowsToSend[idx][key] = value;
                }
              });
            }
          });
          editedRowsToSend.forEach((row: IDictionary) => {
            row.__is_new = false;
            row.NRINTEGRACARDPME = gridPrincipal?.datasource?.currentRow?.NRINTEGRACARDPME;
          });
          const datasourceDetailCrud = component?.userProperties?.userProperties?.dataSourceDetailCrud || '';
          this.crudController.saveRows(editedRowsToSend, datasourceDetailCrud, grid).then(() => {
            this.gridEditController.defaultState(grid);
            this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
            // this.closeCardTpCard();
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          });
        }
      } else {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public setRef({ row }: IDictionary) {
    if (row.NRINTEGRACARDPME) this.detailIsVisible = true;
  }
  
  public onChangeRef({ component }: IEventParam<Datasource>) {
    if (component.currentRow.NRINTEGRACARDPME
      && !Metadata.getInstance<GridEditable>(`gridPrincipal-${this.collapseCardInUse}`)?.editing) {
      const gridDetail = Metadata.getInstance<GridEditable>(`gridDetail-${this.collapseCardInUse}`);
      
      const conditions = [
        {
          name: 'NRINTEGRACARDPME',
          operator: '=',
          value: component.currentRow.NRINTEGRACARDPME,
        },
      ];
      
      if ((gridDetail?.datasource as IDatasource)?.route === '/find/grupTipoCardRef'
        || (gridDetail?.datasource as IDatasource)?.route === '/find/tipoCardRef') {
        conditions.push({
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        });
      } else if ((gridDetail?.datasource as IDatasource)?.route === '/find/servRef') {
        conditions.push({
          name: 'CDFILIAL',
          operator: '=',
          value: this.appController.getCdCentral(),
        });
      }
      
      gridDetail.datasource.filter = conditions;
      gridDetail.datasource.get();
      // this.cardDetailIsVisible();
    } else {
      this.closeCardDetail();
    }
  }
  
  private cardDetailIsVisible(column: any) {
    const card = Metadata.getInstance<Card>(`detailCard-${this.collapseCardInUse}`);
    const gridCentral: GridEditable = Metadata.getInstance(`gridPrincipal-${this.collapseCardInUse}`);
    try {
      const columnEditable = column?.column?.editable || '';
      if (!gridCentral.editing && !gridCentral.editingNewRows && !columnEditable) {
        card.isVisible = true;
        this.detailIsVisible = true;
      } else {
        card.isVisible = false;
        this.detailIsVisible = false;
        if (gridCentral.editing) this.gridEditController.defaultState(gridCentral);
      }
    } catch (Error) {
      card.isVisible = false;
      this.detailIsVisible = false;
    }
  }

  public gruptipoCardComponentProps = {
    component: 'ZdSelectMultiple',
    dataText: [
      {
        name: 'CDGRUPTIPOCARD',
      },
      'NMGRUPTIPOCARD',
    ],
    formatterDataText: [
      {
        name: 'CDGRUPTIPOCARD',
      },
      'NMGRUPTIPOCARD',
    ],
    dataValueOut: [
      {
        column: 'CDFILCENTPME',
        columnOnGrid: 'CDFILCENTPME',
      },
    ],
    dataValue: 'CDGRUPTIPOCARD',
    returnObject: false,
    dense: true,
    datasource: {
      uniqueKey: 'CDGRUPTIPOCARD',
      route: '/find/searchGruptipocard',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: [
        {
          name: 'P_CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'P_CDGRUPTIPOCARD',
          operator: '=',
          value: 'T',
        },
      ],
    },
    validations: {
      required: {},
    },
  };
  
  public grupServComponentProps = {
    component: 'ZdSelectMultiple',
    dataText: [
      {
        name: 'CDGRUPSERV',
      },
      'NMGRUPSERV',
    ],
    formatterDataText: [
      {
        name: 'CDGRUPSERV',
      },
      'NMGRUPSERV',
    ],
    dataValue: 'CDGRUPSERV',
    returnObject: false,
    dense: true,
    datasource: {
      uniqueKey: 'CDGRUPSERV',
      route: '/find/grupserv',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: [],
    },
    validations: {
      required: {},
    },
  };
  
  public tipoCardComponentProps = {
    component: 'ZdSelectMultiple',
    dataText: [
      {
        name: 'CDTIPOCARD',
      },
      'NMTIPOCARD',
    ],
    formatterDataText: [
      {
        name: 'CDTIPOCARD',
      },
      'NMTIPOCARD',
    ],
    dataValueOut: [
      {
        column: 'CDFILCENTPME',
        columnOnGrid: 'CDFILCENTPME',
      },
    ],
    dataValue: 'CDTIPOCARD',
    returnObject: false,
    dense: true,
    datasource: {
      uniqueKey: 'CDTIPOCARD',
      route: '/find/tpCardapioPorCentralAtivo',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'NRPASTATCARD',
          operator: '=',
          value: '',
        },
      ],
    },
    validations: {
      required: {},
    },
  };
  
  public servicoComponentProps = {
    component: 'ZdSelectMultiple',
    dataText: [
      {
        name: 'CDSERVFILI',
      },
      'NMSERVICO',
    ],
    formatterDataText: [
      {
        name: 'CDSERVFILI',
      },
      'NMSERVICO',
    ],
    dataValueOut: [
      {
        column: 'CDFILIAL',
        columnOnGrid: 'CDFILIAL',
      },
    ],
    dataValue: 'CDSERVFILI',
    returnObject: false,
    dense: true,
    datasource: {
      uniqueKey: 'CDSERVFILI',
      route: '/find/servicoPorUnidade',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'IDTIPOSERV',
          operator: '=',
          value: 'M',
        },
      ],
    },
    validations: {
      required: {},
    },
  };
  
  public rotaComponentProps = {
    component: 'ZdSelectMultiple',
    dataText: [
      {
        name: 'CDROTATRANS',
      },
      'NMROTATRANS',
    ],
    formatterDataText: [
      {
        name: 'CDROTATRANS',
      },
      'NMROTATRANS',
    ],
    dataValueOut: [
      {
        column: 'CDFILIAL',
        columnOnGrid: 'CDFILIAL',
      },
    ],
    dataValue: 'CDROTATRANS',
    returnObject: false,
    dense: true,
    datasource: {
      uniqueKey: 'CDROTATRANS',
      route: '/find/rota',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
      filter: [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
      ],
    },
    validations: {
      required: {},
    },
  };

  public changeCurrentDetail({ component }: IEventParam<Datasource>) {
    if (!this.isFormatting) {
      component.loading = true;
      this.isFormatting = true;
      setTimeout(() => {
        this.formatData({ component });
        this.isFormatting = false;
        component.loading = false;
      }, 50);
    }
  }

  public async formatData({ component }: IEventParam<Datasource>) {
    if (!component.data || component.data.length === 0) return;
  
    const mappedData = await component.data.map((row: IDictionary) => {
      const data: any = {};
      Object.keys(row).forEach((key: any) => {
        if (row[key] && !Array.isArray(row[key])) data[key] = [row[key]];
        else data[key] = row[key];
      });
      return data;
    });
    await component.updateData(mappedData);
  }
  
  public deleteDetail({ component }: IEventParam<Button>) {
    this.gridEditController.deleteCheckedRows({ component }, () => {
      const detail = Metadata.getInstance<GridEditable>(`gridDetail-${this.collapseCardInUse}`);
      const checkedRows = detail.selectedRows.map((row: IDictionary) => {
        Object.keys(row).forEach((key: any) => {
          if (Array.isArray(row[key])) row[key] = row[key][0];
        });
        row.NRINTEGRACARDPME = null;
        return row;
      });
      const datasourceDetailCrud = component?.userProperties?.userProperties?.dataSourceDetailCrud || '';
      LoadingService.show(' ');
      this.crudController.saveRows(checkedRows, datasourceDetailCrud, detail).then(() => {
        this.gridEditController.defaultState(detail);
        this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        DialogService.hide();
        LoadingService.hide();
      });
    });
  }
  
  public setListenerClosestEl() {
    window.addEventListener('click', (e) => {
      const target = e.target || e.srcElement;
      const el = (target as Element)?.closest('.zd-collapse-card');
      if (el?.id?.split('-')[1]) this.collapseCardInUse = el?.id?.split('-')[1];
      // console.warn(this.collapseCardInUse);
    }, true);
  }
  
  public get setFilterContrato() {
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRCONTINTPME',
        operator: '=',
        value: true,
      },
    ];
  }
  
  public get setFilterPreparacao() {
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: 'P',
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: (Config.masks.plateMask as PmeMask).nivelMax,
      },
      {
        name: 'CDPRATEXTPME',
        operator: '=',
        value: true,
      },
    ];
  }
  
  public get setFilterPrato() {
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: 'P',
      },
      {
        name: 'CDNVPRATO',
        operator: '=',
        value: (Config.masks.plateMask as PmeMask).nivelMax,
      },
    ];
  }
  
  public async saveGridWithoutDetail({ component }: IEventParam<Button>) {
    const grid: GridEditable = component.parent as GridEditable;
    const editedRows: IDictionary = grid?.getEditedRows(false, true);

    if (grid?.isGridValid(true)) {
      if (editedRows.length > 0) {
        editedRows.forEach((row: IDictionary) => {
          row.__is_new = false;
        });
        const datasourceName = grid?.userProperties?.userProperties?.datasourceCrud || '';
        LoadingService.show(' ');
        this.crudController.saveRows(editedRows, datasourceName, grid).then(() => {
          this.gridEditController.defaultState(grid);
          // gridServicoDetail.datasource.get();
          this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
      } else {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }

  public deleteGridWithoutDetail({ component }: IEventParam<Button>) {
    this.gridEditController.deleteCheckedRows({ component }, () => {
      const grid: GridEditable = component.parent as GridEditable;
      const datasourceName = grid?.userProperties?.userProperties?.datasourceCrud || '';
      const checkedRows = grid.selectedRows.map((row: IDictionary) => {
        row.__is_new = false;
        if (datasourceName === 'unidade') row.NRCONTINTPME = null;
        else if (datasourceName === 'prato') row.CDPRATEXTPME = null;
        return row;
      });
      LoadingService.show(' ');
      this.crudController.saveRows(checkedRows, datasourceName, grid).then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Excluído com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        DialogService.hide();
        LoadingService.hide();
      });
    });
  }
}
